# -*- coding: utf-8 -*-
"""
:Module:         khoros.utils.tests.test_board_creation
:Synopsis:       This module is used by pytest to verify that the board creation works properly
:Created By:     Jeff Shurtliff
:Last Modified:  Jeff Shurtliff
:Modified Date:  18 Jun 2020
"""

import pytest

from . import resources


def get_required_fields(board_type='forum', all_types=False):
    """This function defines required fields that can be used in other tests.

    :param board_type: The type of board (e.g. ``forum``, ``blog``, etc.) to use
    :type board_type: str
    :param all_types: Defines if required fields for all board types should be returned
    :type all_types: bool
    :returns: A tuple or list of tuples with the required fields for one or more board types
    """
    board_id, board_title = 'test-board', 'Test Board'
    if all_types:
        field_data = []
        board_types = ['blog', 'contest', 'forum', 'idea', 'qanda', 'tkb']
        for board_type in board_types:
            field_data.append((board_id, board_title, board_type))
    else:
        field_data = (board_id, board_title, board_type)
    return field_data


def get_dict_for_required_fields(required_fields):
    """This function places the required fields in a properly formatted dictionary.

    :param required_fields: The board ID, title and type
    :type required_fields: tuple, list, set
    :returns: Dictionary containing the required fields
    """
    field_dict = {
        'id': required_fields[0],
        'title': required_fields[1],
        'conversation_style': required_fields[2]
    }
    return field_dict


def verify_data_fields(payload, data_fields):
    """This function checks a dictionary of data fields and values to ensure they match what is in the payload.

    :param payload: The payload for a new board
    :type payload: dict
    :param data_fields: The data fields and corresponding values to check
    :type data_fields: dict
    :returns: Boolean value indicating whether or not the verification checks out
    """
    verified = True
    for field, value in data_fields.items():
        if field not in payload['data']:
            verified = False
            break
        else:
            if payload['data'].get(field) != value:
                verified = False
                break
    return verified


def test_required_fields():
    """This function tests that the payload is structured properly with only the required fields supplied."""
    payload = boards.structure_payload(khoros, 'test-forum', 'Test Forum', 'forum')
    data_fields_to_check = {
        'conversation_style': 'forum',
        'id': 'test-forum',
        'title': 'Test Forum'
    }
    assert verify_data_fields(payload, data_fields_to_check) is True
    return


def test_valid_board_types():
    """This function tests to ensure that the payload for all valid board types gets formatted appropriately."""
    board_types = ['blog', 'contest', 'forum', 'idea', 'qanda', 'tkb']
    board_id, board_title = 'test-board', 'Test Board'
    for board_type in board_types:
        payload = boards.structure_payload(khoros, board_id, board_title, board_type)
        data_fields_to_check = {
            'conversation_style': board_type,
            'id': board_id,
            'title': board_title
        }
        assert verify_data_fields(payload, data_fields_to_check) is True
    return


def test_no_arguments():
    """This function tests to ensure that a TypeError is raised if no arguments are passed to the function."""
    with pytest.raises(TypeError):
        boards.structure_payload()
    return


def test_invalid_board_type():
    """This function tests to ensure an ``InvalidNodeTypeError`` exception is raised for an invalid board type."""
    with pytest.raises(exceptions.InvalidNodeTypeError):
        boards.structure_payload(khoros, 'test-board', 'Test Board', 'group-hub')
    return


def test_description():
    """This function tests the description argument to ensure it gets formatted properly for all board types."""
    description = "This is a description of the new board."
    required_fields = get_required_fields(all_types=True)
    for fields in required_fields:
        board_id, board_title, board_type = fields
        payload = boards.structure_payload(khoros, board_id, board_title, board_type, description)
        expected_fields = get_dict_for_required_fields(fields)
        expected_fields['description'] = description
        assert verify_data_fields(payload, expected_fields) is True
    return


# Import modules and initialize the core object
boards, exceptions = resources.import_modules('khoros.structures.boards', 'khoros.errors.exceptions')
khoros = resources.initialize_khoros_object()



# -*- coding: utf-8 -*-
"""
:Module:            khoros.utils.helper
:Synopsis:          Module that allows the khoros library to leverage a helper configuration file
:Usage:             ``from khoros.utils import helper``
:Example:           ``helper_settings = helper.get_settings('/tmp/helper.yml', 'yaml')``
:Created By:        Jeff Shurtliff
:Last Modified:     Jeff Shurtliff
:Modified Date:     13 Jul 2020
"""

import json

import yaml

from .. import errors
from . import log_utils
from .core_utils import get_file_type

# Initialize logging within the module
logger = log_utils.initialize_logging(__name__)


def import_helper_file(file_path, file_type):
    """This function imports a YAML (.yml) or JSON (.json) helper config file.

    .. versionchanged:: 3.3.0
       A log entry was added to report when the helper file has been imported successfully.

    .. versionchanged:: 2.2.0
       Changed the name and replaced the ``yaml.load`` function call with ``yaml.safe_load`` to be more secure.

    :param file_path: The file path to the YAML file
    :type file_path: str
    :param file_type: Defines the file type as either ``yaml`` or ``json``
    :type file_type: str
    :returns: The parsed configuration data
    :raises: :py:exc:`FileNotFoundError`, :py:exc:`khoros.errors.exceptions.InvalidHelperFileTypeError`
    """
    with open(file_path, 'r') as cfg_file:
        if file_type == 'yaml':
            helper_cfg = yaml.safe_load(cfg_file)
        elif file_type == 'json':
            helper_cfg = json.load(cfg_file)
        else:
            raise errors.exceptions.InvalidHelperFileTypeError()
    logger.info(f'The helper file {file_path} was imported successfully.')
    return helper_cfg


def _convert_yaml_to_bool(_yaml_bool_value):
    """This function converts the 'yes' and 'no' YAML values to traditional Boolean values."""
    true_values = ['yes', 'true']
    if _yaml_bool_value.lower() in true_values:
        _bool_value = True
    else:
        _bool_value = False
    return _bool_value


def _get_connection_info(_helper_cfg):
    """This function parses any connection information found in the helper file.

    .. versionchanged:: 2.2.0
       Removed one of the preceding underscores in the function name
    """
    _connection_info = {}
    _connection_keys = ['community_url', 'tenant_id', 'default_auth_type']
    for _key in _connection_keys:
        if _key in _helper_cfg['connection']:
            _connection_info[_key] = _helper_cfg['connection'][_key]

    # Parse OAuth 2.0 information if found
    if 'oauth2' in _helper_cfg['connection']:
        _connection_info['oauth2'] = _get_oauth2_info(_helper_cfg)

    # Parse session authentication information if found
    if 'session_auth' in _helper_cfg['connection']:
        _connection_info['session_auth'] = _get_session_auth_info(_helper_cfg)
    return _connection_info


def _get_oauth2_info(_helper_cfg):
    """This function parses OAuth 2.0 information if found in the helper file.

    .. versionchanged:: 2.2.0
       Removed one of the preceding underscores in the function name
    """
    _oauth2 = {}
    _oauth2_keys = ['client_id', 'client_secret', 'redirect_url']
    for _key in _oauth2_keys:
        if _key in _helper_cfg['connection']['oauth2']:
            _oauth2[_key] = _helper_cfg['connection']['oauth2'][_key]
        else:
            _oauth2[_key] = ''
    return _oauth2


def _get_session_auth_info(_helper_cfg):
    """This function parses session authentication information if found in the helper file.

    .. versionchanged:: 2.2.0
       Removed one of the preceding underscores in the function name
    """
    _session_auth = {}
    _session_info = ['username', 'password']
    for _key in _session_info:
        if _key in _helper_cfg['connection']['session_auth']:
            _session_auth[_key] = _helper_cfg['connection']['session_auth'][_key]
        else:
            _session_auth[_key] = None
    return _session_auth


def _get_discussion_styles(_helper_cfg):
    """This function defines (when present in the configuration) the enabled discussion styles in the environment.

    :param _helper_cfg: The configuration parsed from the helper configuration file
    :type _helper_cfg: dict
    :returns: List of enabled discussion styles in the environment
    """
    _discussion_styles = ['blog', 'contest', 'forum', 'idea', 'qanda', 'tkb']
    if 'discussion_styles' in _helper_cfg:
        if isinstance(_helper_cfg.get('discussion_styles'), list):
            _discussion_styles = _helper_cfg.get('discussion_styles')
    return _discussion_styles


def _get_construct_info(_helper_cfg):
    """This function parses settings that can be leveraged in constructing API responses and similar tasks.

    .. versionchanged:: 2.8.0
       The function was refactored to leverage the :py:func:`khoros.utils.helper._collect_values` function.

    .. versionchanged:: 2.2.0
       Removed one of the preceding underscores in the function name

    :param _helper_cfg: The configuration parsed from the helper configuration file
    :type _helper_cfg: dict
    :returns: A dictionary with the key value pair for the ``prefer_json`` key if found in the config file
    """
    _top_level_keys = ['prefer_json']
    return _collect_values(_top_level_keys, _helper_cfg)


def _collect_values(_top_level_keys, _helper_cfg, _helper_dict=None, _ignore_missing=False):
    """This function loops through a list of top-level keys to collect their corresponding values.

    .. versionadded:: 2.8.0

    :param _top_level_keys: One or more top-level keys that might be found in the helper config file
    :type _top_level_keys: list, tuple, set, str
    :param _helper_cfg: The configuration parsed from the helper configuration file
    :type _helper_cfg: dict
    :param _helper_dict: A predefined dictionary to which the key value pairs should be added
    :type _helper_dict: dict, None
    :param _ignore_missing: Indicates whether or not fields with null values should be ignored (``False`` by default)
    :type _ignore_missing: bool
    :returns: A dictionary with the identified key value pairs
    """
    _helper_dict = {} if not _helper_dict else _helper_dict
    _top_level_keys = (_top_level_keys, ) if isinstance(_top_level_keys, str) else _top_level_keys
    for _key in _top_level_keys:
        if _key in _helper_cfg:
            _key_val = _helper_cfg[_key]
            if _key_val in HelperParsing.yaml_boolean_values:
                _key_val = HelperParsing.yaml_boolean_values.get(_key_val)
            _helper_dict[_key] = _key_val
        else:
            if not _ignore_missing:
                _helper_dict[_key] = None
    return _helper_dict


def get_helper_settings(file_path, file_type='yaml'):
    """This function returns a dictionary of the defined helper settings.

    .. versionchanged:: 2.8.0
       The function was updated to capture the ``translate_errors`` value when defined.

    .. versionchanged:: 2.2.0
       Support was added for JSON-formatted helper configuration files.

    :param file_path: The file path to the helper configuration file
    :type file_path: str
    :param file_type: Defines the helper configuration file as a ``yaml`` file (default) or a ``json`` file
    :type file_type: str
    :returns: Dictionary of helper variables
    :raises: :py:exc:`khoros.errors.exceptions.InvalidHelperFileTypeError`
    """
    # Initialize the helper_settings dictionary
    helper_settings = {}

    if file_type != 'yaml' and file_type != 'json':
        file_type = get_file_type(file_path)

    # Import the helper configuration file
    helper_cfg = import_helper_file(file_path, file_type)

    # Populate the connection information in the helper dictionary
    if 'connection' in helper_cfg:
        helper_settings['connection'] = _get_connection_info(helper_cfg)

    # Populate the construct information in the helper dictionary
    helper_settings['construct'] = _get_construct_info(helper_cfg)

    # Populate the enabled discussion styles in the helper dictionary
    helper_settings['discussion_styles'] = _get_discussion_styles(helper_cfg)

    # Populate the error translation setting in the helper dictionary
    helper_settings.update(_collect_values('translate_errors', helper_cfg, _ignore_missing=True))

    # Return the helper_settings dictionary
    return helper_settings


class HelperParsing:
    """This class is used to help parse values imported from a YAML configuration file."""
    # Define dictionary to map YAML Boolean to Python Boolean
    yaml_boolean_values = {
        True: True,
        False: False,
        'yes': True,
        'no': False
    }



# -*- coding: utf-8 -*-
"""
:Module:         khoros.utils.tests.test_http_headers
:Synopsis:       This module is used by pytest to verify that HTTP headers are formatted appropriately
:Created By:     Jeff Shurtliff
:Last Modified:  Jeff Shurtliff
:Modified Date:  18 Jun 2020
"""

import pytest

from . import resources


def compare_headers(headers):
    """This function compares two HTTP headers to ensure they are the same.

    .. versionadded:: 2.7.4
    """
    control_dict = {
        'content-type': 'application/json',
        'accept': 'application/json',
        'some-list': ['something', 'something-else'],
        'some-integer': 234
    }
    return True if control_dict == headers else False


def test_normalize_headers():
    """This function tests the :py:func:`khoros.api._normalize_headers` function to ensure it works properly.

    .. versionadded:: 2.7.4
    """
    headers = {
        'Content-Type': 'Application/JSON',
        'Accept': 'application/json',
        'Some-List': ['Something', 'Something-Else'],
        'Some-Integer': 234
    }
    headers = api._normalize_headers(headers)
    assert compare_headers(headers) is True
    return


def test_normalize_empty_headers():
    """This function verifies that passing an empty dictionary to :py:func:`khoros.api._normalize_headers`
       returns the same empty dictionary.

    .. versionadded:: 2.7.4
    """
    headers = api._normalize_headers({})
    assert headers == {}
    return


def test_normalize_type_error():
    """This function verifies that passing no arguments to :py:func:`khoros.api._normalize_headers` raises
       a :py:exc:`TypeError` exception.

    .. versionadded:: 2.7.4
    """
    with pytest.raises(TypeError):
        api._normalize_headers()
    return


# Import modules and initialize the core object
api, exceptions = resources.import_modules('khoros.api', 'khoros.errors.exceptions')
khoros = resources.initialize_khoros_object()



# -*- coding: utf-8 -*-
"""
:Module:            khoros.objects.attachments
:Synopsis:          This module includes functions that handle attachments for messages
:Usage:             ``from khoros.objects import attachments``
:Example:           ``payload = format_attachment_payload(titles, file_paths)``
:Created By:        Jeff Shurtliff
:Last Modified:     Jeff Shurtliff
:Modified Date:     17 Jul 2020
"""

import os
import json

from .. import errors
from ..utils import core_utils, log_utils

# Initialize the logger for this module
logger = log_utils.initialize_logging(__name__)


def construct_multipart_payload(message_json, file_paths, action='create'):
    """This function constructs the full multipart payload for a message with one or more attachment.

    .. versionchanged:: 2.8.0
       Support was added for updating existing messages.

    .. versionadded:: 2.3.0

    :param message_json: The message information in JSON format
    :type message_json: dict
    :param file_paths: The full path(s) to one or more attachment (e.g. ``path/to/file1.pdf``)
    :type file_paths: str, tuple, list, set
    :param action: Indicates if the payload will be used to ``create`` (default) or ``update`` a message
    :type action: str
    :returns: The full payload for the multipart API call as a dictionary

              .. note:: See the `Requests Documentation <https://rsa.im/2SvMsya>`_ for added context on the return data.

    :raises: :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`,
             :py:exc:`khoros.errors.exceptions.DataMismatchError`
    """
    file_paths = core_utils.convert_string_to_tuple(file_paths)
    files_payload = get_file_upload_info(file_paths, action)
    if action == 'update':
        message_json['data'].update(format_attachment_payload(file_paths, 'update'))
        full_payload = _format_full_payload('data', message_json['data'], files_payload)
    else:
        message_json['data']['attachments'] = format_attachment_payload(file_paths)
        full_payload = _format_full_payload('api.request', message_json, files_payload)
    return full_payload


def _format_full_payload(_json_field_name, _json_payload, _files_payload):
    """This function formats the full payload for a ``multipart/form-data`` API request including attachments.

    .. versionadded:: 2.8.0

    :param _json_field_name: The name of the highest-level JSON field used in the JSON payload
    :type _json_field_name: str
    :param _json_payload: The JSON payload data as a dictionary
    :type _json_payload: dict
    :param _files_payload: The payload for the attachments containing the IO stream for the file(s)
    :type _files_payload: dict
    :returns: The full payload as a dictionary
    :raises: :py:exc:`TypeError`
    """
    _full_payload = {
        _json_field_name: (None, json.dumps(_json_payload, default=str), 'application/json')
    }
    _full_payload.update(_files_payload)
    return _full_payload


def format_attachment_payload(file_paths, action='create'):
    """This function formats the JSON payload for attachments to be used in an API call.

    .. versionchanged:: 2.8.0
       Support was added for updating existing messages.

    .. versionadded:: 2.3.0

    :param file_paths: The full path(s) to one or more attachment (e.g. ``path/to/file1.pdf``)
    :type file_paths: str, tuple, list, set
    :param action: Indicates if the payload will be used to ``create`` (default) or ``update`` a message
    :type action: str
    :returns: The list of items (in list format) that represent the ``items`` API value
    :raises: :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`,
             :py:exc:`khoros.errors.exceptions.DataMismatchError`
    """
    file_paths = core_utils.convert_string_to_tuple(file_paths)
    if action == 'update':
        payload = _structure_attachments_to_add(len(file_paths))
    else:
        payload = {
            "list_item_type": "attachment"
        }
        list_items = get_list_items(file_paths)
        payload["items"] = list_items
    return payload


def _structure_attachments_to_add(_attachment_count):
    """This function formats the JSON for the ``attachments_to_add`` field when updating existing messages.

    .. versionadded:: 2.8.0

    :param _attachment_count: The number of attachments being added
    :type _attachment_count: int
    :returns: The properly formatted JSON data as a dictionary
    :raises: :py:exc:`TypeError`
    """
    _attachments = []
    if _attachment_count > 0:
        for _count in range(1, (_attachment_count + 1)):
            _attachment = {
                "type": "attachment",
                "field": f"attachment{_count}"
            }
            _attachments.append(_attachment)
    return {"attachments_to_add": _attachments}


def get_list_items(file_paths):
    """This function constructs the ``items`` field for the ``attachments`` API call.

    .. versionadded:: 2.3.0

    :param file_paths: The full path(s) to one or more attachment (e.g. ``path/to/file1.pdf``)
    :type file_paths: str, tuple, list, set
    :returns: The list of items (in list format) that represent the ``items`` API value
    :raises: :py:exc:`khoros.errors.exceptions.MissingRequiredDataError`,
             :py:exc:`khoros.errors.exceptions.DataMismatchError`
    """
    file_paths = core_utils.convert_string_to_tuple(file_paths)
    list_items, count = [], 1
    for path in file_paths:
        item = {
            "type": "attachment",
            "field": f"attachment{count}",
            "filename": f"{os.path.basename(path)}"
        }
        list_items.append(item)
        count += 1
    return list_items


def get_file_upload_info(file_paths, action='create'):
    """This function constructs the binary file(s) portion of the multipart API call.

    .. versionchanged:: 2.8.0
       Support was added for updating an existing message.

    .. versionadded:: 2.3.0

    :param file_paths: The full path(s) to one or more attachment (e.g. ``path/to/file1.pdf``)
    :type file_paths: str, tuple, list, set
    :param action: Indicates if the payload will be used to ``create`` (default) or ``update`` a message
    :type action: str
    :returns: A dictionary with the file upload information for the API call
    """
    file_paths = core_utils.convert_string_to_tuple(file_paths)
    files, count = {}, 1
    for path in file_paths:
        # TODO: Dynamically define the MIME type
        if action == 'update':
            files[f'attachment{count}'] = open(path, 'rb')
        else:
            files[f'attachment{count}'] = (f'{os.path.basename(path)}', open(path, 'rb'))
        count += 1
    return files



# -*- coding: utf-8 -*-
"""
:Package:           khoros
:Synopsis:          This is the ``__init__`` module for the khoros package
:Usage:             ``from khoros import Khoros``
:Example:           ``khoros = Khoros(community_url='https://community.example.com', helper='path/to/helper_file.yml')``
:Created By:        Jeff Shurtliff
:Last Modified:     Jeff Shurtliff
:Modified Date:     05 Apr 2020
"""

from . import errors
from .core import Khoros
from .utils import version

__all__ = ['core', 'auth', 'liql', 'errors']

# Define the package version by pulling from the khoros.utils.version module
__version__ = version.get_full_version()

# Display a warning if the running version is not the latest stable version found on PyPI
version.warn_when_not_latest()



# -*- coding: utf-8 -*-
"""
:Module:            khoros.errors.translations
:Synopsis:          Provides more relevant translations for error messages in API responses where possible
:Usage:             ``from khoros.errors import translations``
:Example:           ``error_msg = translations.translate_error(error_msg, khoros_object)``
:Created By:        Jeff Shurtliff
:Last Modified:     Jeff Shurtliff
:Modified Date:     26 Dec 2020
"""

ERROR_TRANSLATIONS = {
    'page.post.error.attachment_bad_extension':
        'The attachment does not have an extension permitted in Community Admin > System > File Attachments'
}


def translate_error(error_msg, khoros_object=None, translate_errors=True):
    """This function translates API response errors into more relevant messages where possible and permitted.

    :param error_msg: The original error message
    :type error_msg: str
    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros], None
    :param translate_errors: Defines if errors can be transmitted (``True`` by default) even with no core object
    :type translate_errors: bool
    :return:
    """
    if translation_enabled(translate_errors, khoros_object):
        error_msg = parse_message(error_msg)
        if error_msg in ERROR_TRANSLATIONS:
            error_msg = ERROR_TRANSLATIONS.get(error_msg)
    return error_msg


def translation_enabled(translate_errors=True, khoros_object=None):
    """This function identifies whether or not error translation is permitted.

    .. versionchanged:: 3.3.0
       Updated ``khoros_object._settings`` to be ``khoros_object.core_settings``.

    :param translate_errors: Defines if errors can be transmitted (``True`` by default) even with no core object
    :type translate_errors: bool
    :param khoros_object: The core :py:class:`khoros.Khoros` object
    :type khoros_object: class[khoros.Khoros], None
    :returns: Boolean value defining if translation is enabled
    """
    if khoros_object and 'translate_errors' in khoros_object.core_settings:
        translate_errors = khoros_object.core_settings.get('translate_errors')
    return translate_errors


def parse_message(error_msg):
    """This function parses error messages when necessary to prepare them for translation.

    :param error_msg: The original error message
    :type error_msg: str
    :returns: The prepared error message
    """
    split_sequences = ['\n', '\r']
    for sequence in split_sequences:
        if sequence in error_msg:
            error_msg = error_msg.split(sequence)[0]
            break
    return error_msg



# -*- coding: utf-8 -*-
"""
:Module:            khoros.auth
:Synopsis:          This module handles authentication-related tasks and operations for the Khoros Community APIs
:Usage:             ``import khoros.auth``
:Example:           ``session_key = khoros.auth(KhorosObject)``
:Created By:        Jeff Shurtliff
:Last Modified:     Jeff Shurtliff
:Modified Date:     26 Dec 2020
"""

import requests

from . import api, errors
from .utils import core_utils, log_utils

# Initialize the logger for this module
logger = log_utils.initialize_logging(__name__)


def get_session_key(khoros_object):
    """This function retrieves the session key for an authentication session.

    .. versionchanged:: 3.3.0
       Updated ``khoros_object._settings`` to be ``khoros_object.core_settings``.

    .. versionchanged:: 2.7.4
       The HTTP headers were changed to be all lowercase in order to be standardized across the library.

    :param khoros_object: The core Khoros object
    :type khoros_object: class[khoros.Khoros]
    :returns: The session key in string format
    """
    community_url = khoros_object.core_settings['community_url']
    username = khoros_object.core_settings['session_auth']['username']
    password = khoros_object.core_settings['session_auth']['password']
    query_string = core_utils.encode_query_string({
        'user.login': username,
        'user.password': password,
        'restapi.response_format': 'json'
    })
    uri = f"{community_url}/restapi/vc/authentication/sessions/login/?{query_string}"
    header = {"content-type": "application/x-www-form-urlencoded"}
    response = requests.post(uri, headers=header)
    if response.status_code != 200:
        if type(response.text) == str and response.text.startswith('<html>'):
            api_error = errors.handlers.get_error_from_html(response.text)
            error_msg = f"The authentication attempt failed with the following error:\n\t{api_error}"
            raise errors.exceptions.SessionAuthenticationError(error_msg)
        raise errors.exceptions.SessionAuthenticationError
    else:
        response = response.json()
        session_key = response['response']['value']['$']
    return session_key


def get_session_header(session_key):
    """This function constructs and returns a proper API header containing the session key for authorization.

    :param session_key: The session key for the authorization session
    :type session_key: str
    :returns: The API in dictionary format
    """
    return {'li-api-session-key': session_key}


def invalidate_session(khoros_object, user_id=None, sso_id=None):
    """This function invalidates an active authentication session.

    .. versionchanged:: 3.3.0
       Updated ``khoros_object._settings`` to be ``khoros_object.core_settings``.

    :param khoros_object: The core Khoros object
    :type khoros_object: class[khoros.Khoros]
    :param user_id: The User ID of the service account (Lithium Registration)
    :type user_id: str, int, None
    :param sso_id: The SSO ID of the service account (Single Sign-On)
    :type sso_id: str, int, None
    :returns: Boolean value defining if the session was invalidated successfully
    """
    session_terminated = False
    payload = {}
    if sso_id:
        payload['sso_id'] = sso_id
    elif user_id:
        payload['id'] = user_id
    else:
        username = khoros_object.core_settings['session_auth']['username']
        user_id = khoros_object.search('id', 'users', f'login = "{username}"')
        user_id = user_id['data']['items'][0]['id']
        payload['id'] = user_id
    query_url = f"{khoros_object.core_settings['v2_base']}/auth/signout"
    headers = {'content-type': 'application/json'}
    response = api.post_request_with_retries(query_url, payload, return_json=True,
                                             auth_dict=khoros_object.auth, headers=headers)
    if response['status'] == "success":
        if response['data']['signed_off_all_sessions'] is True:
            session_terminated = True
            print("The authentication session has been terminated.\n")
    return session_terminated


def get_oauth_authorization_url(khoros_object):
    """This function constructs the authorization URL needed for the OAuth 2.0 Web Application Flow.

    .. versionchanged:: 3.3.0
       Updated ``khoros_object._settings`` to be ``khoros_object.core_settings``.

    :param khoros_object: The core Khoros object
    :type khoros_object: class[khoros.Khoros]
    :returns: The properly encoded authorization URL in string format
    """
    community_url = khoros_object.core_settings['community_url']
    query_string = core_utils.encode_query_string({
        'client_id': khoros_object.core_settings['oauth2']['client_id'],
        'redirect_uri': khoros_object.core_settings['oauth2']['redirect_url'],
        'response_type': 'code',
        'state': core_utils.get_random_string()
    }, no_encode='redirect_uri')
    authorization_url = f"{community_url}/auth/oauth2/authorize?{query_string}"
    return authorization_url


def get_oauth_callback_url_from_user(khoros_object):
    """This function instructs the end-user to visit the authorization URL and provide the full callback URL.

    :param khoros_object: The core Khoros object
    :type khoros_object: class[khoros.Khoros]
    :returns: The callback URL supplied by the end-user
    """
    authorization_url = get_oauth_authorization_url(khoros_object)
    print("Please visit this URL to authorize access: ", authorization_url)
    callback_url = input("Enter the full URL in the address bar after visiting the URL above: ")
    if not callback_url.startswith('http'):
        raise errors.exceptions.InvalidCallbackURLError(val=callback_url)
    return callback_url
